
# q-expansion of newform 855.2.k.g, downloaded from the LMFDB on 14 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 855
weight = 2
poly_data = [49, -42, 43, -8, 7, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0], 1], [[294, -42, 43, -49, 7, -1], 259], [[-49, -215, -1, -35, 5, -6], 259], [[952, -210, 215, 14, 35, -5], 259], [[147, 608, 3, 105, 22, 18], 259]]

hecke_ring_character_values = [[191, [1, 0, 0, 0, 0, 0]], [172, [1, 0, 0, 0, 0, 0]], [496, [-1, 0, 0, -1, 0, 0]]]
aps_data = [[0, 1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0], [0, 0, 0, 1, 0, 0], [1, 0, 0, 0, -1, 0], [2, 0, 0, 0, -1, 0], [5, 0, 0, 5, 0, 0], [0, -1, 0, 0, 0, -2], [-1, -1, 1, 0, 2, -1], [3, 1, -3, 1, -2, 2], [-1, 1, 0, 0, 1, -1], [0, 0, -1, 0, 0, 0], [-3, 0, 2, 0, 5, 0], [0, 3, 0, 2, 0, -1], [0, 2, 0, 0, 0, 1], [1, -1, 2, 2, 1, -1], [2, 3, 1, 6, 4, -4], [0, 1, 0, -2, 0, 1], [5, 2, -4, 3, -2, 2], [8, 0, -2, 6, -2, 2], [0, -2, 0, 8, 0, 3], [0, -1, 0, -7, 0, -2], [0, 2, 0, -8, 0, 2], [2, 0, -3, 0, 0, 0], [-2, 0, -4, -6, -4, 4], [0, -1, 0, 3, 0, -3], [0, -5, 1, -4, -4, 4], [7, 0, -5, 0, -6, 0], [-1, 0, -3, 0, -4, 0], [0, -6, 0, -4, 0, 5], [-6, 0, 6, 0, 7, 0], [-2, 6, -6, -2, 0, 0], [0, -5, 0, -6, 0, 5], [-1, 0, -5, -6, -5, 5], [-1, 0, 2, 1, 2, -2], [0, 1, 0, 3, 0, -2], [-11, 0, -4, 0, 0, 0], [0, 0, 0, 5, 0, -1], [2, 0, 5, 0, 6, 0], [9, 1, 0, 10, 1, -1], [0, 3, 0, 2, 0, -3], [13, 0, 1, 0, 2, 0], [1, 6, -8, -1, -2, 2], [8, 0, -6, 0, 0, 0], [0, -1, 0, -10, 0, -2], [-5, 0, -6, 0, -1, 0], [-1, 0, -3, -4, -3, 3], [0, -4, 0, -12, 0, 4], [0, 9, 0, 5, 0, -3], [-4, 0, 0, 0, 0, 0], [0, 0, 2, 0, 7, 0], [0, 10, 0, -3, 0, -1], [-16, 0, 7, 0, 8, 0], [12, -4, 2, 10, -2, 2], [-8, -1, 8, -1, 7, -7], [8, -3, 2, 7, -1, 1], [0, 6, 0, -4, 0, -2], [0, -8, 0, -13, 0, 2], [0, 1, 0, -3, 0, 0], [6, 0, 2, 0, -8, 0], [4, -4, 3, 3, -1, 1], [0, 2, 0, 5, 0, 3], [-9, 0, 3, 0, 4, 0], [0, -5, 0, 7, 0, 5], [-1, 0, 6, 0, 1, 0], [21, -5, 7, 23, 2, -2], [-3, -2, -1, -6, -3, 3], [2, 0, -2, 0, 3, 0], [0, -8, 0, -20, 0, 6], [0, 4, 0, 0, 0, 3], [-4, 0, -1, 0, 5, 0], [20, 0, -1, 0, 2, 0], [0, -3, 0, 14, 0, -8], [-9, -12, 2, -19, -10, 10], [3, 0, -7, 0, -7, 0], [-11, 0, -5, 0, -1, 0], [0, -12, 0, -16, 0, 7], [-12, -11, 8, -15, -3, 3], [0, 7, 0, 8, 0, -7], [0, -4, 0, -14, 0, -3], [-9, -9, 9, -9, 0, 0], [24, 0, -1, 0, 0, 0], [0, -3, 0, -15, 0, 1], [-15, 5, 2, -8, 7, -7], [-12, -3, 8, -7, 5, -5], [0, -5, 0, -1, 0, 17], [-1, 13, -4, 8, 9, -9], [-17, 0, 5, 0, 8, 0], [-2, 0, 11, 0, -4, 0], [0, 1, 0, -6, 0, 5], [7, 0, 3, 0, 8, 0], [3, 0, 4, 0, -2, 0], [-36, 0, -1, -37, -1, 1], [-26, 0, 1, 0, 0, 0], [0, -3, 0, 25, 0, 6], [0, 6, 0, 8, 0, -3], [16, 1, 0, 17, 1, -1], [-4, -13, 13, -4, 0, 0], [12, 0, -3, 0, -13, 0], [2, 17, -10, 9, 7, -7], [3, -4, 1, 0, -3, 3], [26, 3, 1, 30, 4, -4], [11, -2, 5, 14, 3, -3], [-7, 0, -6, 0, 10, 0], [12, 0, -12, 0, -6, 0], [7, 0, -10, 0, 0, 0], [0, 0, -15, 0, -6, 0], [0, 10, 0, -7, 0, -9], [-2, 4, 10, 12, 14, -14], [7, 10, -10, 7, 0, 0], [6, 0, -2, 0, -6, 0], [-5, 0, 13, 0, 14, 0], [0, 9, 0, 11, 0, -10], [-25, 7, 1, -17, 8, -8], [8, 0, 8, 0, -1, 0], [-2, -8, -6, -16, -14, 14], [0, 11, 0, 4, 0, -16], [0, 7, 0, -12, 0, -5], [9, 0, 5, 0, -11, 0], [-16, 0, 2, 0, -3, 0], [-1, -5, -7, -13, -12, 12], [-11, -15, 12, -14, -3, 3], [4, 0, -10, 0, -3, 0], [-5, 0, -5, 0, -3, 0], [32, 0, -10, 0, -7, 0], [-9, 0, 12, 0, -7, 0], [0, -7, 0, -27, 0, 7], [9, -17, 6, -2, -11, 11], [0, -14, 0, -1, 0, 14], [0, 2, 0, -6, 0, -5], [-18, 0, -6, 0, -11, 0], [0, -19, 0, 6, 0, -2], [0, 14, 0, -8, 0, -1], [7, 5, 2, 14, 7, -7], [0, -8, 0, 23, 0, -4], [-9, 0, 15, 0, 9, 0], [-12, 11, 1, 0, 12, -12], [16, -12, 5, 9, -7, 7], [-20, 0, 1, 0, 0, 0], [-28, 0, 0, 0, -1, 0], [-6, 0, 5, 0, -6, 0], [17, -2, -4, 11, -6, 6], [10, -8, -3, -1, -11, 11], [-16, -4, -8, -28, -12, 12], [-6, 7, -10, -9, -3, 3], [17, 0, -9, 0, -15, 0], [0, 3, 0, -6, 0, 2], [0, -5, 0, -27, 0, 2], [0, -11, 0, 12, 0, 5], [-19, 4, 11, -4, 15, -15], [-13, 0, 15, 0, -1, 0], [0, 1, 0, -20, 0, -13], [6, 0, -16, 0, -10, 0], [13, -10, 17, 20, 7, -7], [-14, 10, -7, -11, 3, -3], [0, 4, 0, -18, 0, 4], [5, 0, 0, 0, 6, 0], [16, 0, -23, 0, -17, 0], [0, -14, 0, 4, 0, 13], [2, 12, -13, 1, -1, 1], [10, 6, -14, 2, -8, 8], [0, 4, 0, -21, 0, -1], [0, 11, 0, -7, 0, -17], [0, 4, 0, -46, 0, -4], [0, -4, 0, -17, 0, 5], [-11, 0, 9, 0, 6, 0], [0, 8, 0, 12, 0, 1], [0, 2, 0, 12, 0, -6], [-7, 4, 6, 3, 10, -10]]
